function [out, vars] = homotopy(K,option)
% HOMOTOPY         Use method of homotopy to solve the polynomial system
%
%The instruction
%  
%   [out,vars,msg] = HOMOTOPY(K,para)
%
%  Input : 
%       K -- polynomial system
%       option -- is a struct whose attributes include 'tol', 'refine', 'demand' and 'nonradical' 
%                       attribute 'tol' denotes tolerance
%                       attribute 'refine' denotes we refine the  sample roots or not, 
%                           and must be logical, if ture, we will refine
%                       attribute 'demand' denotes our method to construct a square polynomial 
%                           system when the original system is not square, if the value is 
%                                           'roots_exsitence', we will use method [construct_regular] to construct, or
%                                           'each_component', we will use minor to construct.
%                       attribute 'nonradical' denotes if we fail to obtain a verified root, 
%                           we will use nonradical method to construct a new polynomial system to 
%                           verify sample point again.
%                                           
% Output :
%       out -- results
%       vars -- variables

% Yijun. Zhu, 31 March 2013
%
out = [];
vars = [];
K = vpa(K); % In case of error, expand fraction into decimal
param.tol = option.tol;

% Initialize toRefine
param.toRefine = false;
if isfield(option,'refine')
        param.toRefine = option.refine;
end

% Initialize 'each'
param.each = false;
if isfield(option,'demand')
    if strcmp(option.demand,'each_component')
        param.each = true;
    elseif strcmp(option.demand,'roots_existence')
        param. each = false;
    else
        errorMsg = ['There is no such demand named [', option.demand ,']; Please try [each_component] or [roots_existence]'];
        error(errorMsg);
    end    
end

% Initialize 'isnonradical'
param.isnonradical = false;
if isfield(option, 'nonradical')
    if islogical(option.nonradical)
        param.isnonradical = option.nonradical;
    else
        errorMsg = ['Option [nonradical] must be logical, only true and false is valid'];
        error(errorMsg);
    end
end

% Construct system in Lagrange or Minor
if param.each
    p =  consRegByMinor(K);
    % Replace '0' polynomial
    systems = expand(repZeroPoly(p{1}));
    systems = {systems};
else
    p = construct_regular(K);
    systems = p;
end

for i = 1 : length(systems)
     [out, vars] = homoverify(systems(i), param);     
     if ~isempty(out)
         break;
     end     
end

end % end function

function [out, vars] = homoverify(system, param)
% HOMOVERIFY         The verification of homotopy method
%
% Yijun. Zhu, 31 March 2013
tol = param.tol;
each = param.each;
isnonradical = param.isnonradical;
toRefine = param.toRefine;

% Call hom4ps, and get sample points and char type polynomials
[roots,ff,orders] = myhom4ps(system);
vars = sym(orders);

% Refine sample points when 'toRefine' is true
if toRefine
    newroots = refineRoots(roots,system,vars);
    roots = newroots;    
end

disp(['The number of approximate solutions :       ', int2str(length(roots))]);
% celldisp(roots);


% Verify
[out,terflag,count,remroots] = verifyroots(roots,ff,each);
roots = remroots;

if isempty(out)  || each %If there is no verified sample points or we need every point in component, we continue to verify
    tmpout = verifyRootsByViss(roots,system{:},vars,tol);
    out = [out,tmpout];
    if isempty(out) && isnonradical
%         disp('************** enter nonradical **************' )
        newK = nonradical_poly(system{:},vars);
        if each
            param.demand = 'each_component';
        end
        param.refine = param.toRefine;
        [out, vars] = homotopy(newK,param);
            
        % check every sample points in 'out'
        newout = {};
        count = 1;
%         celldisp(out)
        for i = 1 : length(out)
            e = subs(system,vars,out{i}(1:length(vars),1));
            maxe = max(e);          
            if abs(maxe) < 1E-8
                newout{count} = out{i}; 
                count = count + 1;
            end
        end
        out = newout;
    end
end
end

function newLL = refineRoots(LL,f,vars)
%  Refine sample points
%
%  Input : 
%       LL, -- sample points
%       f -- polynomials
%       newLL -- variables
% Output :
%       newsol -- refined sample points
%
options=optimset('Display','off','TolFun',1e-8);   % Option to display output
newLL = {};
for i = 1 : length(LL)
    % refine one single sample point
    x0 = LL{i};
    objp = 0;
    for j = 1 : length(vars)
        objp = objp + vars(j)^2;
    end
    conp = f{:};
    [x,fval] = fmincon(@(x)myobjfun(x,objp,vars),x0,[],[],[],[],[],[],@(x)myconfun(x,conp,vars),options); % search "Constrained Nonlinear Optimization Examples" in matlab for further imformation
    newLL{i} = x;
end
end

function p = repZeroPoly(p)
% Replace '0' polynomial
repInd = 1;
for i = 1 : length(p)
    pstr = char(p(i));
    if strcmp(pstr,'0')
        p(i) = p(repInd);
        repInd = repInd + 1;
    end
end
end