function [X] = AFPC_BB_moment(n,A,b,maxinneriter,tol,eta,...
                            continuation,schur_type,fast_eig_ratio)
%%**********************************************************************
%%  This is a Matlab implementation of an accelerated fixed point continuation 
%%  method with Barzilai-Borwein technique (AFPC-BB) for solving the nuclear
%%  norm regularized semidefinite least squares problem: 
%%
%%  min_X {mu ||X||_* + 1/2 || A(X) - b ||_2^2}
%%  where X is a positive semidefinite matrix. 
%%
%%  
%% Input: 
%%        n            = size of the matrix. 
%%        r            = the rank of the matrix.
%%        A,num        = function handle the evaluation A(X).
%%        maxinneriter = maximum number of iterations per inner loop.
%%        eta          = rational number <1 to determine how much to decrease
%%                       mu at every step. e.g. eta=1/4.
%%        continuation = 1/0 to determine whether to use continuation technique
%%                       (i.e. a series of mu, aka the "outer loop", until mu = mu_final). 
%%        schur_type   = 'full'or 'partial'.
%%
%% Output: 
%%        X    = the numerical Gram matrix.
%%        iter = the number of iterations over all inner and outer loops.
%%         
%% 
%% Reference:
%%   "The Minimum-Rank Gram Matrix Completion via Modified Fixed Point
%%     Continuation Method."
%%   Yue Ma and Lihong Zhi
%%
%%  code by Yue Ma  yma@mmrc.iss.ac.cn, November 2010
%% **********************************************************************   
   
   if ~strcmp(schur_type,'full') && ~strcmp(schur_type,'partial')
      error(' schur_type must be ''full'' or ''partial'' '); 
   end

   p = length(b);

   fprintf('p:%d\n',p);
   
   
   time1 = cputime;
    
   maxiter = 1e+5; % maximum iteration number for the outer loop
   maxr = floor(n/10);  
   %fast_eig_ratio = 0.001; % ratio for computing the rank for next iteration
   normb = norm(b);
   
%% estimate Lipschitz constant = largest eigenvalue of Map_A(Adj_A). 
%%
    
      options.tol   = 1e-6; 
      options.issym = true; 
      options.disp  = 0; 
      options.v0    = randn(p,1);
      Lipschitz_const = eigs(@(y)Map_A(A,Adj_A(A,y,n)),p,1,'LM',options); 
      tau = 2/full(Lipschitz_const);  
      taumax = 5*tau;
      taumin = 1e-3*tau;
%%
%% initial mu and target mu

   temp = Adj_A(A,b,n);  
   
   if n < 500
          nrm2Adjb = norm(temp);
   else
          T = laneig(temp,3); 
          nrm2Adjb = T(1);
   end
   mu_final = 1e-4*nrm2Adjb;
   
   if continuation
       mu = eta*nrm2Adjb;
   else 
       mu = mu_final;
   end
   if mu < mu_final, mu = mu_final; end
%%  
%% Initialization
         
   X = zeros(n,n);
   Xold = X;
   t = 1;
   extra_rank = 0; 
   violate_num = 0;
   norm_old = inf;
   numin = 0;
   
%% main loop
%%
   fprintf('AFPC-BB:\n');
   fprintf('Iteration begins:\n');
   fprintf('*******************************************************\n');
   
   for iter = 1:maxiter
          
          if mu == mu_final
                 maxinneriter = 50000;
          end

          if (mu > mu_final)
             eigtol = 0.1*min([1e-3,1e-2*mu]);  
          else
             eigtol = 0.1*min([1e-5,1e-1*tol,1e-2*mu]); 
          end

          if iter == 1
              G = tau*temp;
          else
             t     = (1+sqrt(1+4*told^2))/2;
             beta  = (told-1)/t;
             Y    = (beta+1)*X - beta*Xold; 
             AY_b  = Map_A(A,Y)-b;
             Grad  = Adj_A(A,AY_b,n);   
             G     = Y - tt*Grad;  
          end
          Xold = X;
          
          if iter ==1
              if n < 1000 
                     [U, T] = schur(G);  
              else
                     [U, T] = laneig(G,maxr,'LM',eigtol);
              end
          else
              if  strcmp(schur_type,'full')
                  [U, T] = schur(G);
              else
                  Te_old = Te(Te>0); mr = max(Te_old);
                  kk = length(find(Te_old > mr*fast_eig_ratio)); 
                  kk = max(1,min(kk+extra_rank,n));  %disp(kk);
                  G = (G+G')/2; 
                  [U, T] = laneig(G,kk,'LM',eigtol); 
              end
              
          end
                    
          nu = tau*mu; 
          sigma = diag(T);      
          ind = find(sigma > nu); r = length(ind);         
          Te = diag(sigma(ind)-nu); 
          Ue = U(:,ind);                       
          X = Ue*Te*Ue'; 
          
          rX = X-Xold;
          norm_new = norm(rX,'fro');
          normXold = max(norm(Xold,'fro'),1);
          residualX = norm_new/normXold;
          
          AX_b = Map_A(A,X)-b;
          normAX1_b = norm(AX_b);   
          residualA = normAX1_b/normb;  
          
          numin = numin +1;
%           fprintf('***************************\n');
%           fprintf('Iteration:%d\n',numin);
%           fprintf('Rank:%d\n',r);
%           fprintf('Residual_X:%6.8e\n',residualX);
%           fprintf('The relative error of AX-b is %6.8e\n',residualA);
          %fprintf('The nuclear norm is %6.8f\n',norm(diag(Te),1));
          %no need to compute in large scale examples
          told = t; 
          
          if iter ~= 1 
              if norm_new > norm_old
                  violate_num = violate_num + 1;
                  if violate_num >= 10
                      extra_rank = 1;
                      violate_num = 0;
                  else
                      extra_rank = 0;
                  end
              end
          end
         
          norm_old = norm_new;
        
          
          if residualA > 1e5
              disp('Divergence!');
              break
          end
          
          %% check stopping criterion
          if residualA < tol || numin >= maxinneriter || residualX < 1e-5
              numin = 0;
%               fprintf('*******************************************************\n');
              if mu == mu_final
                  eTime = cputime - time1; 
                  %fprintf('Rank:%d\n',r);
                  %fprintf('Residual_X:%6.8e\n',residualX);
                  %fprintf('The relative error of AX-b is %6.8e\n',residualA);
                  fprintf('The number of iteration is %d\n',iter);
                  fprintf('The computing time is %f\n\n',eTime);
                  return
              end
              %% update mu
              mu = max(mu*eta,mu_final);
          end
          
          % update the step size  by Barzilai-Borwein technique
              ArX = Map_A(A,rX);
              rg = Adj_A(A,ArX,n);
              tr_rXrg = sum(sum(rX.*rg));
              tr_rgrg = sum(sum(rg.*rg));
              tt = tr_rXrg/tr_rgrg; 
              tt = max(taumin,min(tt,taumax));      
              
%            % check
%              MM1 = A'*A;            
%              mm = size(MM1,1);
%              MM = eye(mm)-tt*MM1;
%              che = norm(full(MM));
%              disp(che);
%              

   end
%%
%%
%-------------------------------------------------------------------------%
%                            Subfunctions
%-------------------------------------------------------------------------%

   function y = Map_A(A,X)
%   y = A(X)    
       [m1,m2] = size(X);
       x = reshape(X,m1*m2,1);
       y = A*x;
       
       
%-------------------------------------------------------------------------%

function X = Adj_A(A,y,n)
%    X=A^*(y)
      x = A'*y;
      if length(x)==n^2
          X = reshape(x,n,n);
      end 
      X = X+X';
      for i = 1:n
          X(i,i) = X(i,i)/2;
      end
%%
