function [out, vars, solution] = momentsdp(symK,option, t)
% MOMENTSDP         Use method of momentsdp to solve the polynomial system
%
%The instruction
%  
%   [out,vars,msg] = MOMENTSDP(symK, option, t)
%
%  Input : 
%       symK -- polynomial system
%       option -- is a struct whose attributes include 'maxinneriter', 'tol', 'eta', 'continuation', 'schur_type', 'fast_eig_ratio', 
%                       'refine', 'demand', 'curve', 'nonradical', canonical_projection and 'loop'.
%                       attributes 'maxinneriter', 'tol', 'eta', 'continuation', 'schur_type' and 'fast_eig_ratio' 
%                           are used for calling function [AFPC_BB_moment].
%                       attribute 'tol' also denotes tolerance.
%                       attribute 'refine' denotes we refine the  sample roots or not, 
%                           and must be logical. If ture, we will refine the sample roots.
%                       attribute 'demand' denotes our method to construct a square polynomial 
%                           system when the original system is not square, if the value is 
%                                           'fix', we will fix variable(s) to construct, or
%                                           'nullspace', we will use null vector to construct, or
%                                           'regular', we will use method [construct_regular] to construct.
%                       attribute 'curve' denotes our method to construct a square polynomial system 
%                           when ths original system is the special case with 2 variables and 1 polynomial. 
%                           If its value is 'minor', we will use method 'minor' to construct.
%                       attribute 'canonical_projection' denotes if project variables.
%                       attribute 'loop' denotes the maximum iterations(default is 5) to call probabilistic algorithm 'nullspaceConstruct'.
%                    
% 
% Output :
%       out -- results (ie., verified roots, and roots' order  is consistent with variables' order
%       vars -- variables, maybe include auxiliary variable(s)
%       msg -- related information

% Yijun. Zhu, 28 March 2013
%

solution = -1;
out = {};
% Check the specific method
if isfield(option,'method')
    if ~strcmp(option.method,'fix') && ~strcmp(option.method,'nullspace') && ~strcmp(option.method,'regular')
        errorMsg = ['There is no such method named [', option.method ,']; Please try [fix], [nullspace] or [regular]'];
        error(errorMsg);
    end
end

% Initialize 'toRefine'
toRefine = false;
if isfield(option,'refine')
    if isa(option.refine,'logical')
        toRefine = option.refine;
    else
        error('option.refine should be logical');
    end
end

% Initialize 'toProject'
toProject = false;
origSymK = symK;
if isfield(option,'canonical_projection')
    if isa(option.canonical_projection,'logical')
        toProject = option.canonical_projection;
    else
        error('option.canonical_projection should be logical');
    end
end
% Project variables
if toProject
    strvars = findsym(symK);
    vars = sym(['[',strvars,']']);
    if isfield(option, 'fixedVars')
        fixedNum = length(option.fixedVars) - 1;
    elseif length(symK) == 1
        fixedNum = length(vars) - length(symK);
        fixedNum = min(length(vars) - 2, fixedNum); % ensure remain number of variables is greater than or equal to 2
        fixedNum = max(fixedNum, 0); % ensure 'fixedNum' is greater than or equal to zero
    else
        fixedNum = length(vars) - length(symK);
    end
    fixedVals = myrandom(-5,5,1,fixedNum);
    fixedVars = vars(myrandperm(length(vars),fixedNum))
    symK = subs(symK,fixedVars,fixedVals);
end


% Transform polynomials from sym type to mpol type 
[mpolK,strvars] = sym2mpol(symK); % the sequence of variables is in line with the sequence of definition of mpol type variables 
vars =  sym(strvars); 
for i = 1 : length(mpolK)
    K(i) = [mpolK(i)==0];
end
K = K.';

%The input problem is square or not
if length(symK) == length(vars)
    isSquare = true;
else
    isSquare = false;
end

% Initialize 'curve'
curve = '';
if isfield(option,'curve') && ~isSquare
    if strcmp(option.curve,'minor')
        if (length(symK)~=1) || (length(vars)~=2)
            error('only can use [option.curve = minor] when one polynomial with two variables');
        else
            curve = option.curve;
        end
    end
end

% Generate equality matrix by using moment sdp
[b,A_eq] = mymsdp(K,t);

% Generate sample point 
num_var = length(vars);
x_extract = gensampleroots(num_var,t,A_eq,b,mpolK,option)
                    
if ~isempty(x_extract)
    for i = 1 : size(x_extract,2)
        radius{i} = subs(symK,vars,x_extract(:,i));
    end
    celldisp(radius)
end


% Refine sample point
if toRefine
    new_extract = refineRoots2(x_extract,symK,vars)
    if ~isempty(new_extract)
        for i = 1 : size(new_extract,2)
            radius{i} = subs(symK,vars,new_extract(:,i));
        end
        celldisp(radius)
    end
    x_extract = new_extract;
end
 
% To fix 
if ~isfield(option,'projectCount')
        option.projectCount = 0;
end
option.projectCount = option.projectCount + 1;
if option.projectCount <= 10
    if isempty(x_extract) && toProject
        [out, vars, solution] = momentsdp(origSymK,option, t);
        return;
    end
end



% Return when a square problem without sample point
if isempty(x_extract) && isSquare
    msg = '';
    return;
end

% Another data struct of sample points
LL = {};
for i = 1 : size(x_extract,2)
    LL{i} = x_extract(:,i);
end


% Verification Part
tol = option.tol; % tolerance
solution = 0; 
count = 0; % number of results
if ~isSquare
    disp('The system is NOT square !')
    if strcmp(curve, 'minor') && ~isempty(x_extract)  % Special Case :  one polynomial with two variables
        solution = 4;
        [out, count] = minorVerification(symK, vars, x_extract);        
    elseif (~isfield(option,'method') || (isfield(option,'method') && strcmp(option.method,'fix')))  % Method One: fix sample point, and construct  a square problem, and then verify
        if ~isempty(x_extract)            
            solution = 1;
             [out,count] = fixVerification(symK, vars, x_extract);    
        else
            disp('cannot use method [fix] when sample roots is empty');
        end
    else
         count = 0; 
    end
    if 0 == count
        if (~isfield(option,'method') || (isfield(option,'method') && strcmp(option.method,'nullspace')))  % Method Two: use nullspace method to construct a square problem, and then verify
            if ~isempty(x_extract)   
                solution = 2;
                [out, count] = nullspaceVerification(symK,vars,LL,option);
            else
                disp('cannot use method [nullspace] when sample roots is empty');
            end
        end
        if 0 == count
            if (~isfield(option,'method') || (isfield(option,'method') && strcmp(option.method,'regular')))  % Method Three: use regular method to construct a square problem,  and then verify
                solution = 3;
                [out, vars] = regularVerification(symK, t, option);
            end
        end % end if 
    end
else % if  square
    ff = symp2char(symK);
    ff = repvar4inline(ff,vars);
    [out,flag,count] = verifyroots(LL,ff);
    if 0 == count
        out  = verifyRootsByViss(LL,symK,vars,tol);
        if isempty(out) % positive dimension
%             disp('******* Positive dimension case *******')
            out = nullspaceVerification(symK,vars,LL,option);
        end
    end
end
 
% Compute the result when projection
if toProject
    if isempty(out)    
        if isfield(option,'fixedVars') && length(option.fixedVars) == length(fixedVars)
            return;
        end
        option.fixedVals = fixedVals;
        option.fixedVars = fixedVars;
        option.projectCount = 0;
        [out, vars, solution] = momentsdp(origSymK,option, t);
    else        
        for i = 1 : length(out)
            out{i} = [out{i}(1:length(vars),:);[fixedVals(:), fixedVals(:)];out{i}(length(vars)+1:end,:)];
        end       
        vars = [vars,fixedVars];
    end
end

end % end function

function newsol = refineRoots2(sol, p, vars)
%  Refine sample points
%
%  Input : 
%       sol -- sample points
%       p -- polynomials
%       vars -- variables
% Output :
%       newsol -- refined sample points
%
options=optimset('Display','off','TolFun',1e-8);   % Option to display output
newsol = [];
for i = 1 : size(sol,2) 
    % refine one single sample point
    x0 = sol(:,1);
    objp = 0;
    for j = 1 : length(vars)
        objp = objp + vars(j)^2;
    end
    conp = p(:);
    [x,fval] = fmincon(@(x)myobjfun(x,objp,vars),x0,[],[],[],[],[],[],@(x)myconfun(x,conp,vars),options); % search "Constrained Nonlinear Optimization Examples" in matlab for further imformation
    newsol = [newsol,x];
end
end





