function [out, vars,msg] = verify(K,option,varargin)
% VERIFY         Verify the polynomial system K by homotopy or momentsdp
%
%The instruction
%  
%   [out,vars,msg] = VERIFY(K,option,varargin)
%
%  Input : 
%       K -- polynomial system 
%       option -- is a struct.
%                        Attribute [solver] (= 'moment_sdp' or 'homotopy') is essential. 
%                           It decides which verification method to call. 
%                        Attributes [maxinneriter], [tol], [eta], [continuation], [schur_type] 
%                           and [fast_eig_ratio] can be default.
%                        Other attibutes are optional. Details see also momentsdp, homotopy.
%       t -- denotes order. When option.solver = 'homotopy', t is necessary.
% Output :
%       out -- results
%       vars -- variables
%       msg -- related information
% 
% See also  momentsdp, homotopy

% Yijun. Zhu, 31 March 2013
%
msg = '';

if isfield(option, 'solver')
    if strcmp(option.solver,'moment_sdp') || strcmp(option.solver, 'momentsdp') 
        solver = 'momentsdp';
    elseif strcmp(option.solver, 'homotopy')
        solver = 'homotopy';
    else
        error(['There is no such solver [', option.solver, ']. Please try [moment_sdp] or [homotopy] !']);
    end
else
    error('Please choose a solver !')
end

if strcmp(solver,'momentsdp')    
    %Default
    if ~isfield(option, 'maxinneriter')
        option.maxinneriter = 10;
    end
    if ~isfield(option, 'tol')
        option.tol = 1e-3;
    end
    if ~isfield(option, 'eta')
        option.eta = 1/4;
    end
    if ~isfield(option, 'continuation')
        option.continuation = 1;
    end
    if ~isfield(option, 'schur_type')
        option.schur_type = 'partial';
    end
     if ~isfield(option, 'fast_eig_ratio')
        option.fast_eig_ratio = 0.01;
     end
    
    if nargin == 3
        t = varargin{1};
        %Call momentsdp
        [out,vars, solution] = momentsdp(K,option,t);
    else
        error('Input variables is not enough!');
    end
elseif strcmp(solver,'homotopy')
    %Default
    if ~isfield(option, 'tol')
        option.tol = 1e-7;
    end
    %Call homotopy
    [out, vars] = homotopy(K,option);
end


disp('***************************************** ')
if isempty(out)
    disp('Cannot find solution!');
else
    if exist('solution', 'var')
        switch solution
            case 0
                disp('Variables order:');
                disp(vars.');
                msg = 'regular system';
            case 1
                disp('Variables order:');
                disp(vars.');
                disp('Solution 1: fix variables');
                msg = ' fix variables';
            case 2
                disp('Variables order:');
                disp(vars.');
                disp('Solution 2: use svd to construct new polynomials');
                msg = 'use svd to construct new polynomials';
            case 3
                disp('Solution 3: construct regular polynomial group');
                msg = 'construct regular polynomial group';
            case 4 
                disp('Variables order:');
                disp(vars.');
                disp('Solution 4: special case:  one polynomial with two variables');
                msg = 'special case ----  one polynomial with two variables';
        end
    end
end

end